
/*
 * $Header: /cvs/gnome/gperfmeter/src/access/factory.h,v 1.5 2004/11/15 17:12:49 richb Exp $
 *
 * Copyright 2002-2004 Sun Microsystems Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef __FACTORY_H__
#define __FACTORY_H__

#include <atk/atk.h>
#include <gtk/gtk.h>
#include "pangoaccessiblefactory.h"

void setup_factory(void);

#endif /* __FACTORY_H__ */
