
/*
 * $Header: /cvs/gnome/gperfmeter/src/callbacks.h,v 1.6 2004/11/15 17:12:49 richb Exp $
 *
 * Copyright (c) 1990-2004 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */

#include <gnome.h>

gboolean DismissAbout(GtkWidget *, GdkEvent *, gpointer);
gboolean DismissPropsCB(GtkWidget *, GdkEvent *, gpointer);

void PropsButtonCB(GtkButton *, gpointer);
void PropsLimitCB(GtkEditable *, gpointer);
void PropsMeterCB(GtkToggleButton *, gpointer);
void PropsTextAdd(GtkEditable *, gchar *, gint, gint *, gpointer);
void PropsToggleCB(GtkToggleButton *, gpointer);

GtkWidget *MakeColorPicker(gchar *, gchar *, gchar *, gint, gint);
